texture gDepthBuffer : DEPTHBUFFER;
matrix gProjectionMainScene : PROJECTION_MAIN_SCENE;
sampler SamplerDepth = sampler_state
{
    Texture     = (gDepthBuffer);
    AddressU    = Clamp;
    AddressV    = Clamp;
};
struct PSInput
{
  float4 Position : POSITION0;
  float2 TexCoord0 : TEXCOORD0;
};
float FetchDepthBufferValue( float2 uv )
{
    float4 texel = tex2D(SamplerDepth, uv);
#if IS_DEPTHBUFFER_RAWZ
    float3 rawval = floor(255.0 * texel.arg + 0.5);
    float3 valueScaler = float3(0.996093809371817670572857294849, 0.0038909914428586627756752238080039, 1.5199185323666651467481343000015e-5);
    return dot(rawval, valueScaler / 255.0);
#else
    return texel.r;
#endif
}
float Linearize(float posZ)
{
    return gProjectionMainScene[3][2] / (posZ - gProjectionMainScene[2][2]);
}
float3 UnitToColor24New(in float depth) 
{
    const float3 scale	= float3(1.0, 256.0, 65536.0);
    const float2 ogb	= float2(65536.0, 256.0) / 16777215.0;
    const float normal	= 256.0 / 255.0;
    float3 unit	= (float3)depth;
    unit.gb	-= floor(unit.gb / ogb) * ogb;
    float3 color = unit * scale;
    color = frac(color);
    color *= normal;
    color.rg -= color.gb / 256.0;

    return color;
}

float4 pack(float Depth)
{
    if (Depth < 0.05) return float4(Depth/0.05, 0,0,1);
    else if (Depth < 0.25) return float4(0, (Depth-0.05)/0.2,0,1);
    else return float4(0, 0,(Depth-0.25)/0.75,1);
}

float unpack(float4 color)
{
    return (color.r * 0.05) + (color.g * 0.2 + 0.05) + (color.b * 0.25 + 0.25) + (color.a * 0.5 + 0.5); 
}

float4 PS_Example( PSInput In ) : COLOR
{
    float BufferValue = FetchDepthBufferValue( In.TexCoord0.xy );
    float Depth = Linearize( BufferValue );
    Depth *= 0.01f;
    return pack(Depth);
}
technique yes_effect
{
    pass P0
    {
        PixelShader  = compile ps_2_0 PS_Example();
    }
}
technique no_effect
{
    pass P0
    {
    }
}